import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;

/**
 * A subclass of JInternalFrame that contains
 * a RenderedImage.
 */
class ImageInternalFrame extends JInternalFrame {

        /**
         * The image loaded from the file.
         *  This image is not updated when scaling/zooming,
         *   etc. occur
         */
        protected RenderedImage image;

        private ImageViewer parent;
        private int index;
        private String scaleFactor;

	private String filename;

	public ImageInternalFrame(String title, boolean resizable,
                                    boolean closable, boolean maximizable,
                                     boolean iconifiable, int _index,
                                      ImageViewer imageViewer) {

        	super(title, resizable, closable, maximizable, iconifiable);
        	filename = title;
        	parent = imageViewer;
		index = _index;
		scaleFactor = "1x";
		addInternalFrameListener(
			     new InternalFrameAdapter() {
				public void internalFrameClosing(InternalFrameEvent e) {
                                	parent.decrementOpenFrameCount(filename);
				}
				public void internalFrameActivated(InternalFrameEvent e) {
                                        parent.updateSelectedFrame(index, scaleFactor);
                                }
			     }
		);

        }

        public String getFilename() {

        	return filename;

        }

	public RenderedImage getImage() {

		return image;

	}

        public void setImage(RenderedImage im) {

        	image = im;

        }

        public int getIndex() {

        	return index;

        }

        public String getScaleFactor() {

		return scaleFactor;

        }

        public void setScaleFactor(String s) {

        	scaleFactor = s;

        }

        public void setIFrameSize(int width, int height, ImageInternalFrame iFrame) {

		if ( (width < IVConstants.minWidth) ||
		       (height < IVConstants.minHeight) ) // below min height or width
                   iFrame.setSize( IVConstants.minWidth, IVConstants.minHeight );
                else if ( (width < IVConstants.maxWidth) && (height < IVConstants.maxHeight) )
                   iFrame.setSize( width, height );
                else if ( (width >= IVConstants.maxWidth) &&
                            (height >= IVConstants.maxHeight) ) // exceeds max width and height
                   iFrame.setSize( IVConstants.maxWidth, IVConstants.maxHeight );
                else if (width >= IVConstants.maxWidth) // exceeds max width
                   iFrame.setSize( IVConstants.maxWidth, height );
                else if (height >= IVConstants.maxHeight) // exceeds max height
                   iFrame.setSize( width, IVConstants.maxHeight );

	}

}
