import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;

class ImageLoaderThread extends Thread {

	private ImageViewer iv;
	private File file;
	private File[] files;
	private Vector fileVector;
	private boolean multiFiles;
	private int ct, numFiles;

	public ImageLoaderThread(ImageViewer parent, Vector _files) {

		super();
		iv = parent;
		fileVector = _files;
		ct = 0;
		numFiles = fileVector.size();
		multiFiles = true;

	}

	public ImageLoaderThread(ImageViewer parent, File[] _files) {

		super();
		iv = parent;
		files = _files;
		ct = 0;
		numFiles = files.length;
		multiFiles = true;

	}

	public ImageLoaderThread(ImageViewer parent, File _file) {

		super();
		iv = parent;
		file = _file;
		multiFiles = false;

	}

	public void run() {

		if (multiFiles)
		   openMultiFiles();
		else
		   openFile(file);


        }

        private void openFile(File file) {

		String filename = file.getAbsolutePath();

		iv.setStatus( "Loading " + filename + "..." );

        	// load the image
                BufferedImage bufferedImage = null;
                PlanarImage planarImage = JAI.create("fileload", filename );
                try {
                     bufferedImage = planarImage.getAsBufferedImage();
                }
                catch(Exception e) {
                     iv.setStatus("Unable to open " + filename);
                     return;
                }

        	// add the image to the scroll pane
        	DisplayJAI imagePanel = new DisplayJAI(bufferedImage);
        	JScrollPane scrollPane = new JScrollPane(imagePanel);

		ImageInternalFrame frame = iv.addToFrame(scrollPane, file);
                frame.setImage(bufferedImage); // set the frame's image

                int width = bufferedImage.getWidth() + 40;
                int height = bufferedImage.getHeight() + 40;

                doOpen(frame, width, height);

        }

        private void openMultiFiles() {

        	if (fileVector != null) {

        	   for (int i = 0; i < fileVector.size(); i++) {
                       ct++;
                       openFile( (File)( fileVector.get(i) ) );
                   }

        	}
        	else {

        	   for (int i = 0; i < files.length; i++) {
                       ct++;
                       openFile(files[i]);
                   }

                }

        }

        private void doOpen(ImageInternalFrame _frame, int _width, int _height) {

		final ImageInternalFrame frame = _frame;
		final int width = _width;
		final int height = _height;

		Runnable open = new Runnable() {
          	   public void run() {
                       iv.openFrame(frame, width, height); // Runs on the event-dispatching thread.
                       iv.setDefaultStatus();
                       if (multiFiles) {
                          int progress = ct * 100;
                          iv.setProgress( progress/numFiles );
                       }
                   }
      		};
                SwingUtilities.invokeLater(open);

        }

}
