import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;

public class ImagePreview extends JComponent
                          implements PropertyChangeListener {

    /**
     * The maximum size, in bytes, to not create a
     * saved-to-disk thumbnail for an image preview.
     */
    public static final long MAX_FILE_SIZE = 6000;

    /**
     * The name of the directory to store thumbnails in
     * (relative to the location of the full-sized image).
     */
    public static String THUMBNAIL_DIR = ".jimageviewpics";

    private static Scale scale;

    private ImageIcon thumbnail = null;
    private File file = null;
    private int width;
    private ImageViewer parent;
    private ImageFilter imageFilter;
    private boolean cacheThumbnails;
    private boolean createThumbsInBackground;
    private boolean imageFilterCreated = false;

    public ImagePreview(ImageViewer _parent) {
        parent = _parent;
        setMinimumSize(new Dimension(150, 150));
        setPreferredSize(new Dimension(150, 160));
        setMaximumSize(new Dimension(200, 300));
        width = 140;
        scale = new Scale(parent, true);
    }

    public ImagePreview(JFileChooser fc, ImageViewer _parent) {
        parent = _parent;
        setPreferredSize(new Dimension(150, 160));
        fc.addPropertyChangeListener(this);
        width = 140;
        scale = new Scale(parent, true);
    }

    public void enableThumbnailCaching(boolean enable) {

    	cacheThumbnails = enable;

    }

    /**
     * Call only for JFileChooser, not for FilePanel
     */
    public void enableBackgroundThumbsCreation(boolean enable) {

	createThumbsInBackground = enable;

    }


    public ImageIcon getImageIcon(String filename) {

	ImageIcon thumb = null;

	file = new File(filename);
    	thumb = loadImage();
    	while (thumb == null)
    	      loadImage();
        return thumb;

    }

    /**
     * Called by the JList of files in FilePanel
     * to load a new image preview.
     */
    public void updateImage(String filename) {

    	file = new File(filename);
	if ( isShowing() ) {
    	   loadImage();
    	   repaint();
        }

    }

    public ImageIcon loadImage() {

        if (file == null)
            return null;

        long fileSize = file.length(); // get size of file

        // if file too big or not a GIF, JPEG, or PNG,
        //  look for a thumbnail in .jimageviewpics subdirectory of file's directory
        if ( (fileSize > MAX_FILE_SIZE) || Utils.requiresJAI(file) ) {

           // get the name of the thumbnail
           String tName = file.getParent() + ImageViewer.slashS +
                           THUMBNAIL_DIR + ImageViewer.slashS + file.getName();
           File tFile = new File(tName);

           if ( tFile.exists() ) {
              thumbnail = new ImageIcon(tName);
              return thumbnail;
           }
           else {
                  if (cacheThumbnails) {
                     try {
                       String tNailName = scale.createThumbnail(file, tFile, width);
                       if (thumbnail != null)
                          thumbnail = new ImageIcon(tNailName);
                       return thumbnail;
                     }
                     catch (Exception e) {}
                  }
           }

        }

        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon.getIconWidth() > width)
           thumbnail = new ImageIcon( tmpIcon.getImage().getScaledInstance(width,
                                          -1, Image.SCALE_DEFAULT) );
        else
           thumbnail = tmpIcon;

        return thumbnail;

    }

    public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();
        if (prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
            file = (File) e.getNewValue();
            if (isShowing()) {
                loadImage();
                repaint();
            }
        }
        /*else if (prop.equals(JFileChooser.DIRECTORY_CHANGED_PROPERTY)) {
            File currDir = (File) e.getNewValue();
            System.out.println(currDir);
            if ( createThumbsInBackground && isShowing() ) {
                   JFileChooser fc = parent.getFileChooser();
                   File[] files = currDir.listFiles();
                   Vector fileElems = new Vector();
                   Utils.getFilesAndDirs(files, fileElems);
                   String filename;
                   for (int i = 0; i < files.length; i++) {
                       filename = files[i].getAbsolutePath();
                       fileElems.add(i, filename);
                   }
		   ThumbnailThread tt = new ThumbnailThread(fileElems, currDir, parent);
		   tt.start();
	    }
        }*/
    }

    public void paintComponent(Graphics g) {
        if (thumbnail == null) {
            loadImage();
        }
        if (thumbnail != null) {
            int x = getWidth()/2 - thumbnail.getIconWidth()/2;
            int y = getHeight()/2 - thumbnail.getIconHeight()/2;

            if (y < 0) {
                y = 0;
            }

            if (x < 5) {
                x = 5;
            }
            thumbnail.paintIcon(this, g, x, y);
        }
    }
}
