import java.io.*;
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.JPEGEncodeParam;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.beans.*;
import java.net.URL;

/*
   Copyright 2001 Joseph McMurry

   Released under the GPL (version 2)
    (see GPL.txt for more info)
*/

/**
 * @author Joseph McMurry <BR>
 * 3/18/2001              <BR>
 * JImageView application <BR>
 * Allows viewing of GIF, JPEG, PNG, TIF, and BMP (Win95 type) images <BR>
 * @version 0.4      <BR>
 * @since JDK 1.2
 */
public class ImageViewer extends JFrame
                         implements PropertyChangeListener {

	private JDesktopPane desktopPane;
	private ImageInternalFrame frame;
	private JFileChooser fileChooser;
	private FileDialog fileDialog;
	private JComboBox scaleChoice;
	private PrefsDialog prefsDialog;
	private ViewPopupMenu popup;
	private JToolBar toolBar;
	private JPanel contentPane;
	private JSplitPane splitPane;
	private FilePanel filePanel;
	private JMenuItem ocFilePanel;
	private ViewMenu viewMenu;
	private JLabel status;
	private JProgressBar progressBar;
	private JPanel statusPanel;

	private boolean useSwingFileChooser;
	private boolean previewImages;
	private boolean useFileFilters;
	private boolean useFileTypeIcons;
	private boolean useFilePanel;

	/**
	 * If true, images open in the previous session are opened
	 * again when the program is restarted.
	 */
	private boolean openFromPrev;

	private String useJavaLF;

	private OpenFrameList openFrameList;
	private int numFramesCreated; // the number of internal frames created so far
	private int openFrameCount; // the number of open internal frames
	private int scaleType; // the type of scaling/zooming algorithm to be used
	private Properties properties;

	/**
	 * Cached copy of System.getProperty("file.separator")
	 */
	public static String slashS;

	/**
	 * Location of configuration file.
	 */
	public static String configFileWPath;
	public static String[] scaleSizes;

	private ImageViewer(String[] args) {

		numFramesCreated = 0;
		openFrameCount = 0;

		slashS = System.getProperty("file.separator");

                String filesToOpen = loadProperties();

		buildGUI();

		// open the file named on the command line, if any
                if ( args.length > 0 )
                   showImageAI( new File(args[0]) );

                // open files from previous session if enabled
                if (filesToOpen != null)
                   openFiles(filesToOpen);

	}

	private void buildGUI() {

		setLF();

                contentPane = new JPanel();
                contentPane.setLayout( new BorderLayout() );

                desktopPane = new JDesktopPane();

		openFrameList = new OpenFrameList(properties);
                String prop = properties.getProperty(PropertiesConstants.dragModeS);
                if ( !(prop).equals(PropertiesConstants.liveS) )  // if dragMode != live
                   // make dragging internal frames faster
                   desktopPane.putClientProperty("JDesktopPane.dragMode", PropertiesConstants.outlineS);

		createStatusPanel();

                if (useFilePanel) {
                   filePanel = new FilePanel(this, properties);
                   setupSplitPane();
                }
                else  {
                   contentPane.add( desktopPane, BorderLayout.CENTER );
		   contentPane.add( statusPanel, BorderLayout.SOUTH );
		}

		setContentPane(contentPane);

		createMenus();

		createToolBar();
		layoutToolBar();

		openMainWindow();

		if (useFilePanel)
		   filePanel.init();

	}

	private void setLF() {

		try {
		     if ( useJavaLF.equals(PropertiesConstants.javaS) )
                        UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );
                     else if ( useJavaLF.equals(PropertiesConstants.nativeS) ) {
		        String os = System.getProperty("os.name");
			int index = os.indexOf("Linux");
			if (index > -1) // use Motif L&F on Linux
		           UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
                        else
			   UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		     }
                }
                catch (Exception e) {}

	}

	private void createStatusPanel() {

		statusPanel = new JPanel();
		statusPanel.setLayout( new BorderLayout() );
		Border raisedbevel, loweredbevel, compound1;
		raisedbevel = BorderFactory.createRaisedBevelBorder();
                loweredbevel = BorderFactory.createLoweredBevelBorder();
                compound1 = BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
		statusPanel.setBorder(compound1);
		status = new JLabel(IVConstants.jimageviewS);
		progressBar = new JProgressBar();
		statusPanel.add(status, BorderLayout.WEST);
                statusPanel.add(progressBar, BorderLayout.EAST);

	}

	private void layoutToolBar() {

		String toolBarPos = properties.getProperty(PropertiesConstants.toolBarPosS);

                String toolBarLayout;
                if ( toolBarPos.equals(PropertiesConstants.rightS) )
                   toolBarLayout = BorderLayout.EAST;
                else // if (north)
                   toolBarLayout = BorderLayout.NORTH;

       	   	contentPane.add( toolBar, toolBarLayout );

	}

	private void setupSplitPane() {

		JPanel desktopPanel = new JPanel();
		desktopPanel.setLayout( new BorderLayout() );
		desktopPanel.add(desktopPane, BorderLayout.CENTER);
		desktopPanel.add(statusPanel, BorderLayout.SOUTH);

        	splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                               filePanel, desktopPanel);
                splitPane.setOneTouchExpandable(true);
                splitPane.setDividerLocation(170);
                splitPane.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, this);
                contentPane.add(splitPane, BorderLayout.CENTER );

	}

	private void openMainWindow() {

		setTitle(IVConstants.jimageviewS);

		URL iconURL = ClassLoader.getSystemResource("icons/program.gif");
		Image img = Toolkit.getDefaultToolkit().getImage(iconURL);
                setIconImage(img);

	        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

	        String prop = properties.getProperty(PropertiesConstants.fullScreenS);
	        boolean fullScreen = Boolean.valueOf(prop).booleanValue();
		int width, height;

	        if (!fullScreen) {
                   width = Integer.parseInt( properties.getProperty(PropertiesConstants.widthS) );
	           height = Integer.parseInt( properties.getProperty(PropertiesConstants.heightS) );
	        }
	        else { // fullScreen
	           width = (int)d.getWidth();
	           height = (int)d.getHeight();
	        }
	        pack();
	        setSize(width, height);

	        // center the imageViewer window (JFrame) on the screen
	        Rectangle rect = new Rectangle(new Point(0,0), d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

		setVisible(true);

		addWindowListener(
			     new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
				   System.exit(0);
				}
		             }
		);

	}

	public boolean getProp(String propName) {

		String prop = properties.getProperty(propName);
                boolean tf = Boolean.valueOf(prop).booleanValue();

		return tf;

        }

        /**
         * Reads the configuration file
         * into the Properties object.
         */
	private String loadProperties() {

     		properties = new Properties();
     		FileInputStream in = null;
     		BufferedInputStream bis = null;
     		String dir = System.getProperty(IVConstants.userHomeS);
     		configFileWPath = dir+slashS+IVConstants.configFilename;
     		try {
     		     in = new FileInputStream(configFileWPath);
                     bis = new BufferedInputStream(in);
                     properties.load(bis);
	             useJavaLF = properties.getProperty(PropertiesConstants.useJavaLFS);
	             useSwingFileChooser = getProp(PropertiesConstants.useSwingFileChooserS);
	             previewImages = getProp(PropertiesConstants.previewImagesS);
	             useFileFilters = getProp(PropertiesConstants.useFileFiltersS);
	             useFileTypeIcons = getProp(PropertiesConstants.useFileTypeIconsS);
	             useFilePanel = getProp(PropertiesConstants.useFilePanelS);
	             openFromPrev = getProp(PropertiesConstants.openFromPrevS);
	             String prop = properties.getProperty(PropertiesConstants.scaleTypeS);
	             if ( prop.equals(PropertiesConstants.nearestS) )
	                scaleType = Interpolation.INTERP_NEAREST;
	             else
	                scaleType = Interpolation.INTERP_BILINEAR;
	             String filesToOpen;
	             if (openFromPrev)
	                filesToOpen = properties.getProperty(PropertiesConstants.openFrameListS);
	             else
	                filesToOpen = null;
	             return filesToOpen;
	        }
                catch (Exception e) {
                     // assume defaults for config file since one does not exit
                     System.out.println("Configuration file does not exist");
                     setProps();
                     // write default config file to disk
                     try {
                          FileOutputStream out = new FileOutputStream(configFileWPath);
             		  BufferedOutputStream bos = new BufferedOutputStream(out);
                          properties.store(bos, null);
                          bos.close();
                          out.close();
                     }
                     catch (IOException ioe) {
                     	  System.err.println("Properties file could not be saved");
                     }
                }
                return null;

        }

        private void openFiles(String filesToOpen) {

        	Vector files = new Vector();

		StringTokenizer st = new StringTokenizer(filesToOpen, ",");

		while ( st.hasMoreTokens() ) {

		      String token = st.nextToken();

		      if ( (token != null) && ( !token.equals("null") ) )
                         files.add( new File(token) );

                }

                showImageAI(files);

        }

        private void setProps() {

                     useSwingFileChooser = true;
                     previewImages = true;
                     useJavaLF = PropertiesConstants.javaS;
                     useFileFilters = true;
                     useFileTypeIcons = true;
                     useFilePanel = true;
                     scaleType = Interpolation.INTERP_BILINEAR;
        	     properties.setProperty(PropertiesConstants.useSwingFileChooserS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.dragModeS, PropertiesConstants.outlineS);
                     properties.setProperty( PropertiesConstants.imageDirS,
                                              System.getProperty(IVConstants.userHomeS) );
                     properties.setProperty(PropertiesConstants.widthS, "730");
                     properties.setProperty(PropertiesConstants.heightS, "550");
                     properties.setProperty(PropertiesConstants.fullScreenS, IVConstants.falseS);
                     properties.setProperty(PropertiesConstants.toolBarPosS, PropertiesConstants.topS);
                     properties.setProperty(PropertiesConstants.previewImagesS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useJavaLFS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useFileFiltersS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.useFileTypeIconsS, IVConstants.trueS);
		     properties.setProperty(PropertiesConstants.resizeOnScaleS, IVConstants.falseS);
                     properties.setProperty(PropertiesConstants.useFilePanelS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.cacheThumbsS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.createThumbsInBackgroundS, IVConstants.trueS);
                     properties.setProperty(PropertiesConstants.scaleTypeS, PropertiesConstants.bilinearS);
                     properties.setProperty(PropertiesConstants.openFromPrevS, IVConstants.falseS);
		     properties.setProperty(PropertiesConstants.timerDelayS, "2");

        }

	private void createMenus() {

		JMenuBar menuBar = new JMenuBar();

                // create "File" menu
                createFileMenu(menuBar);

                // create "Edit" Menu
                JMenu editMenu = new JMenu("Edit");
                editMenu.setMnemonic(KeyEvent.VK_E);
                createMenuItem("Preferences...", KeyEvent.VK_P,
                                 KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK),
                                  editMenu, new ReflectiveAction(this, "prefsAction") );
                menuBar.add(editMenu);

                // create "View" Menu
                viewMenu = new ViewMenu(menuBar, useFilePanel, filePanel, openFrameList, this);

                createSlideShowMenu(menuBar);

                // create "Options" Menu
                if (useFilePanel) {
                   JMenu optionsMenu = new JMenu("Options");
                   optionsMenu.setMnemonic(KeyEvent.VK_T);
                   ocFilePanel = new JMenuItem(IVConstants.cfp);
		   ocFilePanel.setMnemonic(KeyEvent.VK_C);
                   ocFilePanel.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK));
                   ocFilePanel.addActionListener( new ReflectiveAction(this, "ocFilePanelAction")  );
                   ocFilePanel.setActionCommand(IVConstants.ocFilePanelS);
                   optionsMenu.add(ocFilePanel);
                   menuBar.add(optionsMenu);
                }

                // create "Help" Menu
                JMenu helpMenu = new JMenu("Help");
                helpMenu.setMnemonic(KeyEvent.VK_H);
                createMenuItem("About", KeyEvent.VK_A,
                                KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK),
                                 helpMenu, new ReflectiveAction(this, "aboutAction") );
                menuBar.add( Box.createHorizontalGlue() ); // space help menu to far right
                menuBar.add(helpMenu);

                setJMenuBar(menuBar);

	}

	private void createFileMenu(JMenuBar menuBar) {

		JMenu fileMenu = new JMenu("File");
                fileMenu.setMnemonic(KeyEvent.VK_F);
                createMenuItem("Open...", KeyEvent.VK_O,
                                KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK),
                                 fileMenu, new ReflectiveAction(this, "openAction") );
                createMenuItem("Close", KeyEvent.VK_C,
                                KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK),
                                 fileMenu, new ReflectiveAction(this, "closeAction") );
                createMenuItem("Close All", KeyEvent.VK_A,
                                KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK),
                                 fileMenu, new ReflectiveAction(this, "closeAllAction") );
                fileMenu.addSeparator();
		createMenuItem("Exit", KeyEvent.VK_X,
                                KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK),
                                 fileMenu, new ReflectiveAction(this, "exitAction") );
                menuBar.add(fileMenu);

	}

	private void createSlideShowMenu(JMenuBar menuBar) {

		JMenu slideShowMenu = new JMenu("Slide Show");
		slideShowMenu.setMnemonic(KeyEvent.VK_H);

                if (useFilePanel) {
                   createMenuItem("Slide Show", KeyEvent.VK_H,
                                   KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK),
                                    slideShowMenu, new ReflectiveAction(this, "slideShowAction") );
                   slideShowMenu.addSeparator();
		}

		createMenuItem("Build Custom Slide Show...", KeyEvent.VK_D,
                                KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK),
                                 slideShowMenu, new ReflectiveAction(this, "createCustomSSAction") );
                createMenuItem("Play Custom Slide Show", KeyEvent.VK_P,
                                KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK),
                                 slideShowMenu, new ReflectiveAction(this, "playCustomSSAction") );

                menuBar.add(slideShowMenu);

	}

	/**
	 * Create the popup (right-click) menu.
	 */
        private void createPopupMenu() { popup = new ViewPopupMenu(this); }

	private void createToolBar() {

		toolBar = new JToolBar();

		createButton("icons/Open24.gif", "Open", new ReflectiveAction(this, "openAction") );
		toolBar.addSeparator();

		if (useFilePanel) {
		   createButton("icons/Refresh24.gif", "Refresh File/Directory List",
		                 new ReflectiveAction(filePanel, "currDirAction") );
		   toolBar.addSeparator();
		   createButton("icons/Movie24.gif", "Slide Show",
                                 new ReflectiveAction(this, "slideShowAction") );
                }
                createButton("icons/Play24.gif", "Play Custom Slide Show",
                                 new ReflectiveAction(this, "playCustomSSAction") );
		toolBar.addSeparator();

		createButton("icons/Zoom24.gif", "Scale/Zoom",
		              new ReflectiveAction(viewMenu, "scaleAction") );
		createButton("icons/ZoomIn24.gif", "Zoom In",
		              new ReflectiveAction(viewMenu, "zoomInAction") );
		createButton("icons/ZoomOut24.gif", "Zoom Out",
		              new ReflectiveAction(viewMenu, "zoomOutAction") );

		toolBar.addSeparator();
		createScaleChoice();
		toolBar.addSeparator();

                createButton("icons/Preferences24.gif", "Preferences",
                              new ReflectiveAction(this, "prefsAction") );
                toolBar.addSeparator();

                createButton("icons/About24.gif", "About",
                              new ReflectiveAction(this, "aboutAction") );

	}

	private void createButton(String filename, String toolTipText, ActionListener acListen) {

		URL iconURL = ClassLoader.getSystemResource(filename);
		ImageIcon icon = new ImageIcon(iconURL);
	        JButton button = new JButton(icon);
      	 	button.setToolTipText(toolTipText);
	 	button.addActionListener(acListen);
	 	toolBar.add(button);

	}

	/**
	 * Creates the combo box for scaling an image.
	 */
	private void createScaleChoice() {

		scaleSizes = new String[7];
	 	int i;
	 	int j = 0;
	 	for (i = 4; i > 0; i--) { // create items 4x to 1x
	 	    scaleSizes[j] = ( new Integer(i) ).toString() + IVConstants.xS;
	 	    j++;
	 	}
	 	String div = "/";
	 	for (i = 2; i < 5; i++) { // create items 1/2x to 1/4x
	 	    scaleSizes[j] = IVConstants.oneS + div + ( new Integer(i) ).toString() +
	 	                     IVConstants.xS;
	 	    j++;
	 	}
                scaleChoice = new JComboBox(scaleSizes);
                scaleChoice.setEditable(true);
                scaleChoice.setSelectedIndex(3); // set 1x to be the selected value
                scaleChoice.addActionListener( new ReflectiveAction(viewMenu, "scaleChoiceAction") );
                scaleChoice.setToolTipText("Scale/Zoom");
                scaleChoice.setMaximumSize( new Dimension(80, 25) );
                toolBar.add(scaleChoice);

	}

	public static void createMenuItem(String text, int mnemonic, KeyStroke accelerator,
                                           JMenu menu, ActionListener acListen) {

        	JMenuItem menuItem = new JMenuItem(text);
                menuItem.setMnemonic(mnemonic);
                menuItem.setAccelerator(accelerator);
                menuItem.addActionListener(acListen);
                menu.add(menuItem);

        }

	public void resetDragMode(String dragMode) {

		desktopPane.putClientProperty("JDesktopPane.dragMode", dragMode);

	}

	public void resetFileChooserETC() {

	        useSwingFileChooser = getProp(PropertiesConstants.useSwingFileChooserS);
	        previewImages = getProp(PropertiesConstants.previewImagesS);
	        boolean cacheThumbs = getProp(PropertiesConstants.cacheThumbsS);
	        boolean backgroundCache = getProp(PropertiesConstants.createThumbsInBackgroundS);
	        if (previewImages && (fileChooser != null) ) {
	           ImagePreview ip = (ImagePreview)fileChooser.getAccessory();
	           ip.enableThumbnailCaching(cacheThumbs);
	           ip.enableBackgroundThumbsCreation(backgroundCache);
	        }
	        fileChooser = null; // need to rebuild file chooser next time it's opened

	        useFilePanel = getProp(PropertiesConstants.useFilePanelS);

                String prop = properties.getProperty(PropertiesConstants.imageDirS);
                if ( !prop.endsWith(slashS) )
                   prop += slashS;
                if (filePanel != null) {
                   filePanel.setCurrDirItem(prop);
                   filePanel.enablePreviews(previewImages); // enable/disable previews in filePanel
                   ImagePreview ip = filePanel.getImagePreviewer();
                   ip.enableThumbnailCaching(cacheThumbs);
                   filePanel.enableBackgroundThumbsCreation(backgroundCache);
	        }
	        useJavaLF = properties.getProperty(PropertiesConstants.useJavaLFS);
	        useFileFilters = getProp(PropertiesConstants.useFileFiltersS);
	        useFileTypeIcons = getProp(PropertiesConstants.useFileTypeIconsS);

		prop = properties.getProperty(PropertiesConstants.scaleTypeS);
	        if ( prop.equals(PropertiesConstants.nearestS) )
	           scaleType = Interpolation.INTERP_NEAREST;
	        else
	           scaleType = Interpolation.INTERP_BILINEAR;

	}

	/**
	 * Opens an existing image file.
	 */
	public void openAction() {

		File file;

		if (useSwingFileChooser) {

		   if (fileChooser == null)
		      createFileChooser();

		   int returnVal = fileChooser.showOpenDialog(this); // open the modal file chooser

                   if (returnVal == JFileChooser.APPROVE_OPTION) {

                      if ( useJavaLF.equals(PropertiesConstants.javaS) ) {
                         // multi-selection only works w/ Java L&F
                         File[] files = getSelectedFiles(fileChooser);
                         showImageAI(files/*[i]*/); // use JAI for all images to support scaling
                      }
                      else {
                         file = fileChooser.getSelectedFile();
                         showImageAI(file);
                      }

                   }

                } // end if (useSwingFileChooser)
                else { // native file dialog

		   if (fileDialog == null) {
                      fileDialog = new FileDialog(this,"Open",FileDialog.LOAD);
                      fileDialog.setDirectory( properties.getProperty(PropertiesConstants.imageDirS) );
                   }

		   fileDialog.setFile(Utils.allTypes); // file "filter" for Win32

                   fileDialog.setVisible(true);
                   String filename;
                   if ( (filename = fileDialog.getFile() ) != null) {
                      file = new File( fileDialog.getDirectory() + filename );
                      showImageAI(file);
		   }
                }

	}

	private void createFileChooser() {

		fileChooser = new JFileChooser();
	        useJavaLF = properties.getProperty(PropertiesConstants.useJavaLFS);
		if ( useJavaLF.equals(PropertiesConstants.javaS) )
		   fileChooser.setMultiSelectionEnabled(true);
		String prop = properties.getProperty(PropertiesConstants.imageDirS);
		fileChooser.setCurrentDirectory( new File( prop ) );
		if (useFileFilters)
		   fileChooser.addChoosableFileFilter( new ImageFilter() );
                if (useFileTypeIcons)
                   fileChooser.setFileView( new ImageFileView() );
                if (previewImages) {
                   ImagePreview ip = new ImagePreview(fileChooser, this);
                   fileChooser.setAccessory(ip);
	           boolean cacheThumbs = getProp(PropertiesConstants.cacheThumbsS);
                   ip.enableThumbnailCaching(cacheThumbs);
	           boolean tf = getProp(PropertiesConstants.createThumbsInBackgroundS);
                   ip.enableBackgroundThumbsCreation(tf);
                }

	}

	/**
	 * This is an ugly hack to allow getting multiple files
	 * from a JFileChooser at once.
	 */
	public static File[] getSelectedFiles(JFileChooser chooser) {

        	Container c1 = (Container)chooser.getComponent(3);
		JList list = null;

		while (c1 != null) {
                      Container c = (Container)c1.getComponent(0);
                      if (c instanceof JList) {
                         list = (JList)c;
                         break;
                      }
                      c1 = c;
                }

		Object[] entries = list.getSelectedValues();
                File[] files = new File[entries.length];

		for (int k=0; k<entries.length; k++) {
                    if (entries[k] instanceof File)
                       files[k] = (File)entries[k];
                }

                return files;

        }

        /**
         * Closes all internal frames.
         */
        public void closeAllAction() {

        	ImageInternalFrame frameToClose;

		int len = openFrameList.size();
        	for (int i = 0; i < len; i++) {
        	    frameToClose = (ImageInternalFrame)openFrameList.get(i);
        	    closeIFrame(frameToClose);
        	}

        }

        private void closeIFrame(ImageInternalFrame frameToClose) {

        	if (frameToClose != null) {
		   openFrameCount--;
		   frameToClose.setVisible(false);
		   frameToClose.dispose();
		   frameToClose = null;
		}

        }

	/**
	 * Closes and disposes of the currently selected internal frame.
	 */
	public void closeAction() {

		ImageInternalFrame frameToClose;

		frameToClose = openFrameList.getSelectedFrame();
		if (frameToClose != null) {
		   openFrameCount--;
		   closeIFrame(frameToClose);
		}

	}

	public void exitAction() { System.exit(0); }

	public void prefsAction() {

		if (prefsDialog == null)
                   prefsDialog = new PrefsDialog(this, properties);

		prefsDialog.setVisible(true);

	}

	public void saveAction() {

	}

	private int getTimerDelay() {

		int val;

		String prop = properties.getProperty(PropertiesConstants.timerDelayS);
 		try {
                     val = Integer.parseInt(prop);
        	}
        	catch (NumberFormatException nfe) {
        	     System.err.println("Invalid timer delay, reverting to default (2 seconds)");
        	     val = 2;
        	}

        	return val;

        }

	public void slideShowAction() {

		int td = getTimerDelay() * 1000;
                FullScreenWindow fcw = new FullScreenWindow(td, filePanel.getCurrDir() );
		fcw.setVisible(true);

	}

	/**
	 * Opens the SlideShowBuilder dialog
	 */
	public void createCustomSSAction() {

		SlideShowBuilder ssb = new SlideShowBuilder(this, properties);
		ssb.setVisible(true);

	}

	/**
	 * Plays the custom slide show.
	 */
	public void playCustomSSAction() {

                String fileList;
                fileList = properties.getProperty(PropertiesConstants.customShowS);

                Vector files = parseFileList(fileList);

		int td = getTimerDelay() * 1000;
		FullScreenWindow fcw = new FullScreenWindow(td, files);
		fcw.setVisible(true);

	}

	public Vector parseFileList(String fileList) {

                Vector files = new Vector();

		StringTokenizer st = new StringTokenizer(fileList, ",");

		while ( st.hasMoreTokens() ) {

		      String token = st.nextToken();

		      if ( (token != null) && ( !token.equals("null") ) )
                         files.add(token);

                }

                return files;

	}

        /**
         * Opens the Help->About dialog.
         */
        public void aboutAction() {

        	JIVAboutDialog aboutDialog = new JIVAboutDialog(this);
                aboutDialog.setVisible(true);

        }

	/**
	 * Opens or closes the file panel by expanding/collapsing the split pane
	 */
	public void ocFilePanelAction() { updateSplitPane(true); }

	/**
	 * Property change listener for split pane divider location change
	 */
	public void propertyChange(PropertyChangeEvent evt) { updateSplitPane(false); }

	private void updateSplitPane(boolean resize) {

		int divLoc = splitPane.getDividerLocation();
		String text;
		char key;

		if (divLoc > 10) {
		   if (resize) {
		      splitPane.setDividerLocation(0.0); // close the file panel
		      text = IVConstants.ofp;
		      key = KeyEvent.VK_O;
		   }
		   else {
		      text = IVConstants.cfp;
		      key = KeyEvent.VK_C;
		   }
		   ocFilePanel.setText(text);
		   ocFilePanel.setMnemonic(key);
		}
		else {
		   int lastLoc;
		   if (resize) {
		      lastLoc = splitPane.getLastDividerLocation();
		      splitPane.setDividerLocation(lastLoc); // open the file panel
		      text = IVConstants.cfp;
		      key = KeyEvent.VK_C;
		   }
		   else {
		      text = IVConstants.ofp;
		      key = KeyEvent.VK_O;
		   }
		   ocFilePanel.setText(text);
		   ocFilePanel.setMnemonic(key);
		}

	}

	public ImageInternalFrame getSelectedFrame() {

		return openFrameList.getSelectedFrame();

	}

	public JFileChooser getFileChooser() { return fileChooser; }

	public ViewMenu getViewMenu() { return viewMenu; }

	public JComboBox getScaleChoice() { return scaleChoice; }

	public int getScaleType() { return scaleType; }

        /**
         * Gets the status bar's text.
         */
	public String getStatus() { return status.getText(); }

	/**
	 * Set's the status bar's text.
	 */
	public void setStatus(String text) { status.setText("  "+text); }

	/**
	 * Set's the status bar's text to "JImageView".
	 */
	public void setDefaultStatus() { status.setText(IVConstants.jimageviewS); }

	/**
	 * Set's the progress bar's value.
	 */
	public void setProgress(int val) {

		progressBar.setValue(val);

		if (val == 100)
                   resetProgress();

	}

	public void resetProgress() { progressBar.setValue(0); }

	/**
	 * Opens a JInternalFrame with the image specified
	 * by the File "file".
	 */
        public void showImageAI(File file) {

		try {

		  ImageLoaderThread imageLoader = new ImageLoaderThread(this, file);
                  imageLoader.start();

                } // end try
                catch (Exception e) {
                    JOptionPane.showMessageDialog(this, "Could Not Open Image.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
                }

        }

        /**
	 * Opens a JInternalFrame with the images specified
	 * by the Files array "files".
	 */
        public void showImageAI(File[] files) {

		try {

		  ImageLoaderThread imageLoader = new ImageLoaderThread(this, files);
                  imageLoader.start();

                } // end try
                catch (Exception e) {
                    JOptionPane.showMessageDialog(this, "Could Not Open Image.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
                }

        }

        /**
	 * Opens a JInternalFrame with the images specified
	 * by the Vector "files".
	 */
        public void showImageAI(Vector files) {

		try {

		  ImageLoaderThread imageLoader = new ImageLoaderThread(this, files);
                  imageLoader.start();

                } // end try
                catch (Exception e) {
                    JOptionPane.showMessageDialog(this, "Could Not Open Image.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
                }

        }

        /**
         * Increments openFrameCount, constructs the internal frame,
         * sets its content pane, packs it, and ands an internal
         * frame listener to it.
         */
        public ImageInternalFrame addToFrame(JScrollPane scrollPane, File file) {

		ImageInternalFrame frame;
		frame = new ImageInternalFrame( file.getAbsolutePath(), // title
                                         	  true,                 // resizable
                                                   true,                // closable
                                                    true,               // maximizable
                                                     true,              // iconifiable
                                                      numFramesCreated,
                                                       this );
                numFramesCreated++;
                openFrameList.addElement(frame);
                frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                frame.setContentPane(scrollPane);
                if (popup == null)
                   createPopupMenu();
                frame.addMouseListener(popup);
                frame.pack();

                return frame;

        }

        public void openFrame(ImageInternalFrame frame, int width, int height) {

		frame.setIFrameSize(width, height, frame);

                // tile the internal frames
                int xLoc, yLoc;

                openFrameCount++;
                xLoc = IVConstants.xOffset * ( openFrameCount % 10 );
                yLoc = IVConstants.yOffset * ( openFrameCount % 10 );
                if ( (xLoc < 0) && (yLoc < 0) ) {
                   xLoc = 0;
                   yLoc = 0;
                }
                frame.setLocation(xLoc, yLoc);
                frame.setVisible(true); // necessary as of kestrel
                desktopPane.add(frame);
                try {
                  frame.setSelected(true);
                }
                catch (java.beans.PropertyVetoException e) {}

        }

        public void decrementOpenFrameCount(String filename) {

        	openFrameCount--;
                openFrameList.remove(filename);

        }

        public void updateSelectedFrame(int index, String scaleFactor) {

        	openFrameList.setSelectedFrame(index);

        	// set the scale factor displayed in the combo box to scaleFactor
        	scaleChoice.getEditor().setItem(scaleFactor);

        }

	public static void main(String[] args) {

                ImageViewer imageViewer = new ImageViewer(args);

	}

}
