import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

class PrefsDialog extends JDialog implements ActionListener, ItemListener {

	private JTextField mwWidth, mwHeight;
	private ButtonGroup fcBG, mwDS, mwTBP, mwLF, scaleType;
	private JRadioButton fcUseSwing, fcUseNative, mwOutline, mwLive,
	                      top, right, mwJava, mwDefault, mwNative,
	                       scaleHQ, scaleLQ;
	private JCheckBox previewImages, fullScreen, useFileTypeIcons, fcUseFilePanel,
	                   useFileFilters, resizeIFrames, cacheThumbs, backgroundCache,
	                   openFromPrev;
	private JButton browse, ok, cancel, apply;
	private JFileChooser fileChooser;
	private JTextField dirField;
	private JSlider timerDelay;

	private boolean fileChooserCreated;
	private boolean dialogVisible;

	private Properties properties;
	private ImageViewer parent;


	// constants
	static final String okS = "ok";
	static final String cancelS = "cancel";
	static final String browseS = "browse";

	public PrefsDialog(ImageViewer _parent, Properties prop) {

		super(_parent, "Preferences");
		parent = _parent;
		properties = prop;
		fileChooserCreated = false;
		dialogVisible = false;
		buildGUI();
		pack();
		int width = 380;
		int height = 365;
		setSize(width, height);
		Point p = parent.getLocationOnScreen();
		Dimension d = parent.getSize();
		Rectangle rect = new Rectangle(p, d);
	        setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

	}

	private void buildGUI() {

		JPanel contentPane = new JPanel();
		contentPane.setLayout( new BorderLayout() );

		JTabbedPane tabbedPane = new JTabbedPane();
	        tabbedPane.addTab("File Chooser", null, buildFileChooserPanel(), "Preferences for File Chooser");
	        tabbedPane.addTab("Main Window", null, buildMainWindowPanel(), "Preferences for Main Window");
	        tabbedPane.addTab("Scaling/Thumbnails", null, buildSTPanel(),
	                           "Preferences for Scaling and Thumbnails");
		tabbedPane.addTab("Other", null, buildOtherPanel(), "Other Preferences");

		JPanel buttonsPanel = new JPanel();
		ok = new JButton("Save");
		ok.addActionListener(this);
		ok.setActionCommand(okS);
		cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		cancel.setActionCommand(cancelS);
		buttonsPanel.add(ok);
		buttonsPanel.add(cancel);

		contentPane.add(tabbedPane, BorderLayout.CENTER);
		contentPane.add(buttonsPanel, BorderLayout.SOUTH);

		setContentPane(contentPane);

		// make ok the default button
		getRootPane().setDefaultButton(ok);

	}

	private JPanel buildFileChooserPanel() {

		JPanel fileChooserPanel = new JPanel();

		fileChooserPanel.setPreferredSize( new Dimension(360, 295) );
		fileChooserPanel.setLayout( new GridLayout(3, 1) );

		JPanel type = new JPanel();
		fcBG = new ButtonGroup();
	        fcUseSwing = new JRadioButton("Java");
	        fcUseNative = new JRadioButton("Native");
	        fcUseFilePanel = new JCheckBox("Use File Panel?");
	        fcUseFilePanel.setToolTipText("List-based, added to main window (a la GQView)");
	        fcUseSwing.addItemListener(this);
	        fcUseNative.addItemListener(this);
	        fcUseFilePanel.addItemListener(this);
	        fcBG.add(fcUseSwing);
	        fcBG.add(fcUseNative);
                type.setBorder( new TitledBorder("Type") );
	        type.add(fcUseSwing);
	        type.add(fcUseNative);
	        type.add(fcUseFilePanel);

                JPanel optionsPanel = new JPanel();
                optionsPanel.setBorder( new TitledBorder("Options") );
                previewImages = new JCheckBox("Preview Images?");
	        useFileTypeIcons = new JCheckBox("Use File Type Icons?");
	        useFileFilters = new JCheckBox("Use File Filters?");
		optionsPanel.add(previewImages);
                optionsPanel.add(useFileTypeIcons);
                optionsPanel.add(useFileFilters);

		JPanel imageDirPanel = new JPanel();
		imageDirPanel.setBorder( new TitledBorder("Image Directory") );
		dirField = new JTextField(12);
		browse = new JButton("Browse...");
		browse.addActionListener(this);
		browse.setActionCommand(browseS);
		imageDirPanel.add(dirField);
		imageDirPanel.add(browse);

		fileChooserPanel.add(type);
		fileChooserPanel.add(optionsPanel);
		fileChooserPanel.add(imageDirPanel);

		return fileChooserPanel;

	}

	private JPanel buildMainWindowPanel() {

		JPanel mainWindowPanel = new JPanel();
		mainWindowPanel.setPreferredSize( new Dimension(360, 295) );
		mainWindowPanel.setLayout( new GridLayout(4, 1) );

		JPanel sizePanel = new JPanel();
		sizePanel.setBorder( new TitledBorder("Size") );
                sizePanel.add( new JLabel("Width") );
                mwWidth = new JTextField(3);
                sizePanel.add(mwWidth);
                sizePanel.add( new JLabel("   Height") );
                mwHeight = new JTextField(3);
                sizePanel.add(mwHeight);
                sizePanel.add( new JLabel("   ") );
                fullScreen = new JCheckBox("Full Screen?");
                fullScreen.addItemListener(this);
                sizePanel.add(fullScreen);

                JPanel toolBarPanel = buildToolBarPanel();

		JPanel dragPanel = new JPanel();
                dragPanel.setBorder( new TitledBorder("Drag Style") );
                mwDS = new ButtonGroup();
                mwOutline = new JRadioButton("Outline");
                mwLive = new JRadioButton("Live");
                mwDS.add(mwOutline);
                mwDS.add(mwLive);
                dragPanel.add(mwOutline);
                dragPanel.add(mwLive);

                JPanel lfPanel = new JPanel();
                lfPanel.setBorder( new TitledBorder("Look and Feel") );
                mwLF = new ButtonGroup();
                mwJava = new JRadioButton("Java");
                mwDefault = new JRadioButton("Default");
                mwNative = new JRadioButton("Native");
                mwJava.addItemListener(this);
                mwDefault.addItemListener(this);
                mwNative.addItemListener(this);
                mwLF.add(mwJava);
                mwLF.add(mwDefault);
                mwLF.add(mwNative);
                lfPanel.add(mwJava);
                lfPanel.add(mwDefault);
                lfPanel.add(mwNative);

                mainWindowPanel.add(sizePanel);
                mainWindowPanel.add(toolBarPanel);
                mainWindowPanel.add(dragPanel);
                mainWindowPanel.add(lfPanel);

		return mainWindowPanel;

	}

	public JPanel buildToolBarPanel() {

		JPanel toolBarPanel = new JPanel();
                toolBarPanel.setBorder( new TitledBorder("Tool Bar Position") );
                mwTBP = new ButtonGroup();
                top = new JRadioButton("Top");
                right = new JRadioButton("Right");
                top.addItemListener(this);
                right.addItemListener(this);
                mwTBP.add(top);
                mwTBP.add(right);
                toolBarPanel.add(top);
                toolBarPanel.add(right);

                return toolBarPanel;

	}

	public JPanel buildOtherPanel() {

		JPanel otherPanel = new JPanel();
		otherPanel.setPreferredSize( new Dimension(360, 295) );
		otherPanel.setLayout( new GridLayout(2, 1) );

		JPanel slideShowPanel = new JPanel();
		slideShowPanel.setBorder( new TitledBorder("Slide Show") );
		timerDelay = new JSlider(JSlider.HORIZONTAL, 1, 5, 2);
                timerDelay.setMajorTickSpacing(1);
                timerDelay.setSnapToTicks(true);
                timerDelay.setPaintTicks(true);
                timerDelay.setPaintLabels(true);
                String ts = "Timer Delay in seconds";
		timerDelay.setToolTipText(ts);
		slideShowPanel.add( new JLabel(ts) );
		slideShowPanel.add(timerDelay);

		JPanel morePanel = new JPanel();
		morePanel.setBorder( new TitledBorder("Startup") );
                openFromPrev = new JCheckBox("Open images from previous session?");
		openFromPrev.setToolTipText("At startup, open images from previous session?"); 
		morePanel.add(openFromPrev);

		otherPanel.add(slideShowPanel);
		otherPanel.add(morePanel);

		return otherPanel;

	}

	private JPanel buildSTPanel() {

		JPanel stPanel = new JPanel();
		stPanel.setPreferredSize( new Dimension(360, 295) );
		stPanel.setLayout( new GridLayout(2, 1) );

		JPanel scalingPanel = new JPanel();
		scalingPanel.setBorder( new TitledBorder("Scaling") );
		scaleType = new ButtonGroup();
		scaleHQ = new JRadioButton("Higher Quality/Slower");
		scaleHQ.setToolTipText("Bilinear");
		scaleLQ = new JRadioButton("Lower Quality/Faster");
		scaleLQ.setToolTipText("Nearest-Neighbor");
		scaleType.add(scaleHQ);
		scaleType.add(scaleLQ);
		scalingPanel.add(scaleHQ);
		scalingPanel.add(scaleLQ);

		resizeIFrames = new JCheckBox("Resize Image Windows on Scaling/Zooming?");
		scalingPanel.add(resizeIFrames);


		JPanel thumbsPanel = new JPanel();
		thumbsPanel.setBorder( new TitledBorder("Thumbnails") );
		cacheThumbs = new JCheckBox("Cache Thumbnails to Disk?");
		backgroundCache = new JCheckBox("Create Thumbnails in Background?");
		cacheThumbs.setToolTipText("Save Thumbnail Images on Disk?");
		backgroundCache.setToolTipText("Create thumbnails on a separate thread when" +
                                                 " the directory is first scanned?");
                cacheThumbs.addItemListener(this);
		thumbsPanel.add(cacheThumbs);
		thumbsPanel.add(backgroundCache);

		stPanel.add(scalingPanel);
		stPanel.add(thumbsPanel);

		return stPanel;

	}

	public void itemStateChanged(ItemEvent e) {

		Object item = e.getItem();
		int state = e.getStateChange();

		if ( (item == fcUseSwing) && (state == ItemEvent.SELECTED) ) {
		   enableFCOptions(true);
		}
                else if ( (item == fcUseNative) && (state == ItemEvent.SELECTED) ) {
		   enableFCOptions(false);
		}
		else if (item == fullScreen) {
                   if (state == ItemEvent.DESELECTED)
                      enableMWSize(true);
		   else
		      enableMWSize(false);
		}
		else if (item == cacheThumbs) {
		   if (state == ItemEvent.SELECTED)
		      enableBackgroundCache(true);
		   else
                      enableBackgroundCache(false);
		}
		else if ( (item == fcUseFilePanel) && dialogVisible ) {
		   showMessageDialog("File Panel change will not occur until\n" +
                                      "JImageView is restarted.");
	        }
		else if ( (item == mwJava) || (item == mwNative) || (item == mwDefault) ) {
		   if ( (state == ItemEvent.SELECTED) && dialogVisible )
                      showMessageDialog("The Look and Feel change will not occur until\n" +
                                          "JImageView is restarted.");
	        }
	        else if ( (item == top) || (item == right) ) {
	            if ( (state == ItemEvent.SELECTED) && dialogVisible )
                      showMessageDialog("The Tool Bar Position change will not occur until\n" +
                                         "JImageView is restarted.");
                }

	}

	private void showMessageDialog(String message) {

		JOptionPane.showMessageDialog(this, message, "Message", JOptionPane.INFORMATION_MESSAGE);

	}

	private void enableFCOptions(boolean tf) {

		previewImages.setEnabled(tf);
		useFileTypeIcons.setEnabled(tf);
	        useFileFilters.setEnabled(tf);

	}


	private void enableMWSize(boolean tf) {

		mwWidth.setEnabled(tf);
		mwHeight.setEnabled(tf);

	}

	private void enableBackgroundCache(boolean enable) {

		backgroundCache.setEnabled(enable);

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(okS) )
		   okAction();
		else if ( actionCommand.equals(cancelS) )
		   setVisible(false);
		else if ( actionCommand.equals(browseS) )
		   browseAction();

	}

	private void okAction() {

		boolean noErrors;

	        // get preferences from components
	        noErrors = getPrefs();

                if (noErrors) {
	           setVisible(false);
                   saveProperties(); // save to config file

                   // reset all things affected by properties (except those reset on restart)
                   //  ie filechooser/filepanel, screen size, drag mode, etc.
                   parent.resetFileChooserETC();

                   int width = 730;
                   int height = 530;
                   Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
                   if ( fullScreen.isSelected() ) {
                      width = (int)d.getWidth();
	              height = (int)d.getHeight();
		   }
                   else {
                      try {
                           width = Integer.parseInt( mwWidth.getText() );
                           height = Integer.parseInt( mwHeight.getText() );
                      }
                      catch (NumberFormatException nfe) {}
                   }
                   parent.setSize(width, height);
                   Rectangle rect = new Rectangle(new Point(0,0), d);
                   parent.setLocation( (rect.x + ((rect.width - width)/2)),
		                        (rect.y + ((rect.height - height)/2)) );

		   String dragMode;
		   if ( mwOutline.isSelected() )
		      dragMode = PropertiesConstants.outlineS;
		   else
		      dragMode = PropertiesConstants.liveS;
                   parent.resetDragMode(dragMode);

                }

	}

	private boolean getPrefs() {

		String prop = null;
		boolean noErrors;

		noErrors = getFCPrefs(prop);
		if (noErrors)
		   noErrors = getMWPrefs(prop);
		if (noErrors)
		   noErrors = getOtherPrefs(prop);

		return noErrors;

	}


	private boolean getFCPrefs(String prop) {

	        if ( fcUseSwing.isSelected() )
	           prop = IVConstants.trueS;
	        else if ( fcUseNative.isSelected() )
	           prop = IVConstants.falseS;
	        if (prop != null)
	           properties.setProperty(PropertiesConstants.useSwingFileChooserS, prop);

	        setProp( fcUseFilePanel.isSelected(), PropertiesConstants.useFilePanelS );

	        setProp( previewImages.isSelected(), PropertiesConstants.previewImagesS );

	        setProp( useFileTypeIcons.isSelected(), PropertiesConstants.useFileTypeIconsS );

	        setProp( useFileFilters.isSelected(), PropertiesConstants.useFileFiltersS );

	        prop = dirField.getText();
	        if ( !(new File(prop)).isDirectory() ) { // check to see if the image directory is valid
	           JOptionPane.showMessageDialog(this, "Image Directory does not exist.",
	                                          "Error", JOptionPane.ERROR_MESSAGE);
	           return false;
	        }
	        else
	           properties.setProperty(PropertiesConstants.imageDirS, prop);

	        return true;

	}

	private boolean getMWPrefs(String prop) {

                if ( fullScreen.isSelected() ) {
                   prop = IVConstants.trueS;
                   properties.setProperty(PropertiesConstants.fullScreenS, prop);
                }
                else {
                   boolean retval = getMWPrefsFullScreen(prop);
                   if (!retval)
                      return retval;
                }

                if ( top.isSelected() )
                   prop = PropertiesConstants.topS;
                else // if ( right.isSelected() )
                   prop = PropertiesConstants.rightS;
		properties.setProperty(PropertiesConstants.toolBarPosS, prop);

		if ( mwOutline.isSelected() )
		   prop = PropertiesConstants.outlineS;
		else
		   prop = PropertiesConstants.liveS;
		properties.setProperty(PropertiesConstants.dragModeS, prop);

		if ( mwJava.isSelected() )
		   prop = PropertiesConstants.javaS;
		else if ( mwDefault.isSelected() )
		   prop = PropertiesConstants.defaultS;
		else // if ( mwNative.isSelected() )
		   prop = PropertiesConstants.nativeS;
		properties.setProperty(PropertiesConstants.useJavaLFS, prop);

		return true;

	}

	private boolean getMWPrefsFullScreen(String prop) {

		   prop = IVConstants.falseS;
                   properties.setProperty(PropertiesConstants.fullScreenS, prop);
                   int w, h;
                   prop = mwWidth.getText();
                   try {
                        w = Integer.parseInt(prop);
                   }
                   catch (NumberFormatException nfe1) {
                        JOptionPane.showMessageDialog(this, "Width is not a valid number.",
	                                               "Error", JOptionPane.ERROR_MESSAGE);
                   	return false;
                   }
                   properties.setProperty(PropertiesConstants.widthS, prop);
                   prop = mwHeight.getText();
                   try {
                        h = Integer.parseInt(prop);
                   }
                   catch (NumberFormatException nfe2) {
                        JOptionPane.showMessageDialog(this, "Height is not a valid number.",
	                                               "Error", JOptionPane.ERROR_MESSAGE);
	                return false;
                   }
                   properties.setProperty(PropertiesConstants.heightS, prop);

                   return true;

	}

	private boolean getOtherPrefs(String prop) {

		setProp( resizeIFrames.isSelected(), PropertiesConstants.resizeOnScaleS );

		String val;
		if ( scaleLQ.isSelected() )
		   val = PropertiesConstants.nearestS;
		else
		   val = PropertiesConstants.bilinearS;
		properties.setProperty(PropertiesConstants.scaleTypeS, val);

		setProp( cacheThumbs.isSelected(), PropertiesConstants.cacheThumbsS );
		setProp( backgroundCache.isSelected(), PropertiesConstants.createThumbsInBackgroundS);
		setProp( openFromPrev.isSelected(), PropertiesConstants.openFromPrevS);
		int myVal = timerDelay.getValue();
		String valS = String.valueOf(myVal);
		properties.setProperty(PropertiesConstants.timerDelayS, valS);

		return true;

	}

	private void saveProperties() {

		String dir = System.getProperty(IVConstants.userHomeS);
     		String configFileWPath = dir+ImageViewer.slashS+".jimageviewrc";

		try {
                     FileOutputStream out = new FileOutputStream(configFileWPath);
             	     BufferedOutputStream bos = new BufferedOutputStream(out);
                     properties.store(bos, null);
                     bos.close();
                     out.close();
                }
                catch (IOException ioe) {
                     System.err.println("Properties file could not be saved");
                }

	}

	/**
	 * Opens a new dialog so that the
	 * user can select the default image directory.
	 */
	private void browseAction() {

		if (!fileChooserCreated) {
		   fileChooser = new JFileChooser();
     		   fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		   fileChooser.setDialogTitle("Select the Default Image Directory");
		   fileChooserCreated = true;
                }
		String prop = properties.getProperty(PropertiesConstants.imageDirS);
		fileChooser.setCurrentDirectory( new File( prop ) );

		int returnVal = fileChooser.showDialog(this, "Select");

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                   String directory;
                   File file = fileChooser.getSelectedFile();
                   if ( file.isFile() ) // user selected a normal file
                      directory = file.getParent(); // get the directory the file is in
                   else // user selected a directory
                      directory = file.getPath();
                   dirField.setText(directory);
                }

	}

	public void setVisible(boolean visible) {

		if (visible) // set selected values in prefs dialog from properties
                   setValues();
                else
                   dialogVisible = false;
		super.setVisible(visible);

	}

	// the below functions set the values of the checkboxes, radio buttons, etc in the prefs dialog
	//  when the dialog is opened (setVisible(true))

	private void setValues() {

	        String prop = null;
                boolean tf = false;

                setFCTypeValues();
                setFCOptionsValues();
                setFCImageDirValues();

                setMWSizeValues();
                setMWToolBarPosValues();
                setMWDragStyleValues();
                setMWLFValues();

		setOtherScalingValues();
		setOtherThumbsValues();
		setOtherValues();

		dialogVisible = true;

	}

	private void setProp(boolean selected, String propName) {

		String prop;

		if (selected)
	           prop = IVConstants.trueS;
	        else
	           prop = IVConstants.falseS;
	        properties.setProperty(propName, prop);

        }

        private boolean getProp(String propName) {

		String prop = properties.getProperty(propName);
                boolean tf = Boolean.valueOf(prop).booleanValue();

		return tf;

        }

	private void setFCTypeValues() {

                if ( getProp(PropertiesConstants.useSwingFileChooserS) )
                   fcUseSwing.setSelected(true);
                else
                   fcUseNative.setSelected(true);

                fcUseFilePanel.setSelected( getProp(PropertiesConstants.useFilePanelS) );

	}

	private void setFCOptionsValues() {

                previewImages.setSelected( getProp(PropertiesConstants.previewImagesS) );
                useFileTypeIcons.setSelected( getProp(PropertiesConstants.useFileTypeIconsS) );
                useFileFilters.setSelected( getProp(PropertiesConstants.useFileFiltersS) );

	}

        private void setFCImageDirValues() {

        	String prop = properties.getProperty(PropertiesConstants.imageDirS);
        	dirField.setText(prop);

        }

        private void setMWSizeValues() {

        	String prop = properties.getProperty(PropertiesConstants.widthS);
        	mwWidth.setText(prop);
                prop = properties.getProperty(PropertiesConstants.heightS);
                mwHeight.setText(prop);
                if ( getProp(PropertiesConstants.fullScreenS) ) {
                   // disable height and width
                   mwWidth.setEnabled(false);
                   mwHeight.setEnabled(false);
                   fullScreen.setSelected(true);
                }
                else
                   fullScreen.setSelected(false);

        }

        private void setMWToolBarPosValues() {

        	String prop = properties.getProperty(PropertiesConstants.toolBarPosS);
                if ( prop.equals(PropertiesConstants.topS) )
                   top.setSelected(true);
                else // if (right)
                   right.setSelected(true);

        }

        private void setMWDragStyleValues() {

        	String prop = properties.getProperty(PropertiesConstants.dragModeS);
                if ( prop.equals(PropertiesConstants.liveS) )
        	   mwLive.setSelected(true);
        	else // if ( prop.equals(outlineS) )
        	   mwOutline.setSelected(true);

        }

        private void setMWLFValues() {

		String prop = properties.getProperty(PropertiesConstants.useJavaLFS);

                if ( prop.equals(PropertiesConstants.javaS) )
        	   mwJava.setSelected(true);
        	else if ( prop.equals(PropertiesConstants.defaultS) )
                   mwDefault.setSelected(true);
                else
        	  mwNative.setSelected(true);



        }

	private void setOtherScalingValues() {

        	resizeIFrames.setSelected( getProp(PropertiesConstants.resizeOnScaleS) );

        	String prop = properties.getProperty(PropertiesConstants.scaleTypeS);
        	if ( prop.equals(PropertiesConstants.nearestS) )
	           scaleLQ.setSelected(true);
	        else
	           scaleHQ.setSelected(true);

	}

	private void setOtherThumbsValues() {

        	cacheThumbs.setSelected( getProp(PropertiesConstants.cacheThumbsS) );
        	backgroundCache.setSelected( getProp(PropertiesConstants.createThumbsInBackgroundS) );

        }

        private void setOtherValues() {

        	openFromPrev.setSelected( getProp(PropertiesConstants.openFromPrevS) );
        	String prop = properties.getProperty(PropertiesConstants.timerDelayS);
        	int val;
        	try {
                     val = Integer.parseInt(prop);
        	}
        	catch (NumberFormatException nfe) {
        	     System.err.println("Invalid timer delay, reverting to default (2 seconds)");
        	     val = 2;
        	}
        	timerDelay.setValue(val);

        }

}
