import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class ScaleDialog extends JDialog implements ActionListener {

	private JButton ok, cancel;
	private JComboBox scaleChoice;
	private ImageViewer imageViewer;

	private String[] scaleSizes;


        // constants
	private static final String okS = "ok";
	private static final String cancelS = "cancel";

	public ScaleDialog(ImageViewer parent) {

		super(parent, "Scale/Zoom");

		imageViewer = parent;

		scaleSizes = ImageViewer.scaleSizes;

		buildGUI();

		int width = 220;
		int height = 100;
		setSize(width, height);

		// center the dialog over the main window
		Point p = parent.getLocationOnScreen();
		Dimension d = parent.getSize();
		Rectangle rect = new Rectangle(p, d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );

	}

	private void buildGUI() {

		JPanel contentPane = new JPanel();
		contentPane.setLayout( new BorderLayout() );

		JPanel sp = new JPanel();
		createScaleChoice();
		sp.add( new JLabel("Scale Factor: ") );
		sp.add(scaleChoice);

                JPanel bp = createButtons();


                contentPane.add(sp, BorderLayout.NORTH);
                contentPane.add(bp, BorderLayout.SOUTH);

		setContentPane(contentPane);

		// make ok the default button
		getRootPane().setDefaultButton(ok);

	}

	private JPanel createButtons() {

		JPanel p = new JPanel();

		ok = new JButton("Ok");
                ok.addActionListener(this);
                ok.setActionCommand(okS);
                cancel = new JButton("Cancel");
                cancel.addActionListener(this);
                cancel.setActionCommand(cancelS);

                p.add(ok);
                p.add(cancel);

                return p;

	}

	private void createScaleChoice() {

		scaleChoice = new JComboBox(scaleSizes);
		scaleChoice.setEditable(true);
                scaleChoice.setToolTipText("Scale/Zoom");
                scaleChoice.setPreferredSize( new Dimension(80, 25) );

	}

	public void setVisible(boolean visible) {

		if (visible) {
		   ImageInternalFrame selectedFrame = imageViewer.getSelectedFrame();
		   if (selectedFrame != null) {
		      String scaleFactor = selectedFrame.getScaleFactor();
		      scaleChoice.setSelectedItem(scaleFactor);
		      scaleChoice.getEditor().setItem(scaleFactor);
		   }
		}
		super.setVisible(visible);

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals(okS) ) {

		   setVisible(false);

		   ImageInternalFrame selectedFrame = imageViewer.getSelectedFrame();

                   if (selectedFrame != null) {
                      // get the selected item in the scaleChoice (combo box)
                      String scaleString = (String)scaleChoice.getSelectedItem();
                      ImageScalerThread imageScaler = new ImageScalerThread(imageViewer, selectedFrame,
                                                                             scaleString, 0);
                      imageScaler.start();
                      //imageViewer.scale(scaleString, selectedFrame, 0);
                   }

		}

		else if ( actionCommand.equals(cancelS) )
		   setVisible(false);

	}

}
