import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.net.URL;

class SlideShowBuilder extends JDialog
                       implements ActionListener,
                                   MouseListener {

	private ImageViewer parent;
	private Properties properties;
	private JList fileList;
	private DefaultListModel fileListModel;
	private JFileChooser chooser;
	private JButton addFiles, save, cancel;
	private JPopupMenu popup;
	private boolean javaLF;

	public SlideShowBuilder(ImageViewer parent, Properties properties) {

		super(parent, "Custom Slide Show", false);
		this.parent = parent;
		this.properties = properties;
		buildGUI();

	}

	private void buildGUI() {

		JPanel contentPane = new JPanel();
		contentPane.setLayout( new BorderLayout() );

		createPopup();

                JPanel saveCancelPanel = new JPanel();
                save = createButton("Save", "save");
                cancel = createButton("Cancel", "cancel");
                saveCancelPanel.add(save);
                saveCancelPanel.add(cancel);

		contentPane.add( createUpperPanel(), BorderLayout.NORTH);
		contentPane.add(saveCancelPanel, BorderLayout.SOUTH);

		setContentPane(contentPane);

		int width = 260;
		int height = 200;
		setSize(width, height);
		// center the dialog over the main window
		Point p = parent.getLocationOnScreen();
		Dimension d = parent.getSize();
		Rectangle rect = new Rectangle(p, d);
                setLocation( (rect.x + ((rect.width - width)/2)),
		               (rect.y + ((rect.height - height)/2)) );
		pack();
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

	}

	private void createPopup() {

		popup = new JPopupMenu();
		createMenuItem("Add Files...", "addFiles");
		popup.addSeparator();
		createMenuItem("Shift Up", "up");
		createMenuItem("Shift Down", "down");
		popup.addSeparator();
		createMenuItem("Delete", "delete");
		createMenuItem("Delete All", "deleteAll");

	}

	private void createMenuItem(String text, String actionCommand) {

        	JMenuItem menuItem = new JMenuItem(text);
                menuItem.addActionListener(this);
                menuItem.setActionCommand(actionCommand);
                popup.add(menuItem);

        }

	private void loadFileList() {

		fileListModel = new DefaultListModel();

		String fileList = properties.getProperty(PropertiesConstants.customShowS);
		if ( (fileList != null) && ( !( fileList.equals("null") ) ) ) {
		   // parse fileList and add it to the model
		   Vector files = parent.parseFileList(fileList);
		   int size = files.size();
		   for (int i = 0; i < size; i++)
		       fileListModel.addElement( files.get(i) );
		}

	}

	private JPanel createUpperPanel() {

                JPanel upperPanel = new JPanel();

		upperPanel.setLayout( new BorderLayout() );

		loadFileList(); // get the existing file list (if any)

		fileList = new JList(fileListModel);
		fileList.addMouseListener(this);

		JScrollPane scrollPane = new JScrollPane(fileList);

                upperPanel.add(scrollPane, BorderLayout.NORTH);
		upperPanel.add(createInnerPanel(), BorderLayout.SOUTH);

		return upperPanel;

	}

	private JPanel createInnerPanel() {

		JPanel inner = new JPanel();

		addFiles = createButton("Add Files...", "addFiles");
		addFiles.setMaximumSize( new Dimension(100, 30) );
                JButton up = createButton("icons/Up24.gif", "Shift Up", this, "up");
                up.setMargin(new Insets(0,0,0,0));
                JButton down = createButton("icons/Down24.gif", "Shift Down", this, "down");
                down.setMargin(new Insets(0,0,0,0));
                JButton delete = createButton("icons/Delete24.gif", "Delete", this, "delete");
                delete.setMargin(new Insets(0,0,0,0));
                JButton deleteAll = createButton("icons/Remove24.gif", "Delete All", this, "deleteAll");
                deleteAll.setMargin(new Insets(0,0,0,0));
                inner.add(addFiles);
                inner.add(up);
                inner.add(down);
                inner.add(delete);
                inner.add(deleteAll);

                return inner;

	}

	private JButton createButton(String text, String actionCommand) {

	        JButton button = new JButton(text);
	 	button.addActionListener(this);
	 	button.setActionCommand(actionCommand);

	 	return button;

	}

	private JButton createButton(String filename, String toolTipText,
	                              ActionListener acListen, String actionCommand) {

		URL iconURL = ClassLoader.getSystemResource(filename);
		ImageIcon icon = new ImageIcon(iconURL);
	        JButton button = new JButton(icon);
      	 	button.setToolTipText(toolTipText);
	 	button.addActionListener(acListen);
	 	button.setActionCommand(actionCommand);

	 	return button;

	}

	public void actionPerformed(ActionEvent e) {

		String actionCommand = e.getActionCommand();

		if ( actionCommand.equals("addFiles") )
                   addFilesAction();
                else if ( actionCommand.equals("save") )
                   saveAction();
		else if ( actionCommand.equals("cancel") )
		   cancelAction();
		else if ( actionCommand.equals("up") )
                   upAction();
		else if ( actionCommand.equals("down") )
                   downAction();
                else if ( actionCommand.equals("delete") )
                   deleteAction();
                else if ( actionCommand.equals("deleteAll") )
                   deleteAllAction();

	}

	private void addFilesAction() {

		if (chooser == null) {
		   chooser = parent.getFileChooser();
		   if (chooser == null)
		      createFileChooser();
		}

                int returnVal = chooser.showDialog(this, "Select");

                if (returnVal == JFileChooser.APPROVE_OPTION) {

                   if (javaLF) {
                      File[] files = ImageViewer.getSelectedFiles(chooser);
                      for (int i = 0; i < files.length; i++)
		          fileListModel.addElement( files[i].getAbsolutePath() );
                   }
		   else {
                      File file = chooser.getSelectedFile();
                      fileListModel.addElement( file.getAbsolutePath() );
                   }

                }

	}

	private void createFileChooser() {

		chooser = new JFileChooser();

		String useJavaLF = properties.getProperty(PropertiesConstants.useJavaLFS);
		if ( useJavaLF.equals(PropertiesConstants.javaS) ) {
		   chooser.setMultiSelectionEnabled(true);
		   javaLF = true;
		}
		else
		   javaLF = false;
		String prop = properties.getProperty(PropertiesConstants.imageDirS);
		chooser.setCurrentDirectory( new File( prop ) );
		boolean previewImages = getProp(PropertiesConstants.previewImagesS);
	        boolean useFileFilters = getProp(PropertiesConstants.useFileFiltersS);
	        boolean useFileTypeIcons = getProp(PropertiesConstants.useFileTypeIconsS);
		if (useFileFilters)
		   chooser.addChoosableFileFilter( new ImageFilter() );
                if (useFileTypeIcons)
                   chooser.setFileView( new ImageFileView() );
                if (previewImages) {
                   ImagePreview ip = new ImagePreview(chooser, parent);
                   chooser.setAccessory(ip);
	           boolean cacheThumbs = getProp(PropertiesConstants.cacheThumbsS);
                   ip.enableThumbnailCaching(cacheThumbs);
	           boolean tf = getProp(PropertiesConstants.createThumbsInBackgroundS);
                   ip.enableBackgroundThumbsCreation(tf);
                }

	}

	public boolean getProp(String propName) {

		String prop = properties.getProperty(propName);
                boolean tf = Boolean.valueOf(prop).booleanValue();

		return tf;

        }

	/**
	 * move the selected item up in the list
	 */
	private void upAction() {

		move(-1);

	}

	private void move(int pos) {

                int index = fileList.getSelectedIndex();
		int newIndex = index + pos;
		if ( (newIndex > -1) && ( newIndex < fileListModel.size() ) ) {
		    Object elem = fileListModel.get(index);
		    fileListModel.removeElementAt(index);
		    fileListModel.insertElementAt(elem, index + pos);
                }

	}

	/**
	 * move the selected item down in the list
	 */
	private void downAction() {

		move(1);

	}

	/**
	 * Deletes the selected list element(s)
	 */
	private void deleteAction() {

                Object[] elems = fileList.getSelectedValues();
                for (int i = 0; i < elems.length; i++)
                    fileListModel.removeElement(elems[i]);

	}

	private void deleteAllAction() {

		fileListModel.removeAllElements();

	}

	private void saveAction() {

		int size = fileListModel.size();
		StringBuffer files= new StringBuffer();

		for (int i = 0; i < size; i++) {
		    String file = (String)fileListModel.get(i);
                    files.append(file);
                    files.append(',');
		}

		properties.setProperty( PropertiesConstants.customShowS, files.toString() );

		try {
                     FileOutputStream out = new FileOutputStream(ImageViewer.configFileWPath);
                     BufferedOutputStream bos = new BufferedOutputStream(out);
                     properties.store(bos, null);
                     bos.close();
                     out.close();
                }
                catch (IOException ioe) {
                     System.err.println("Custom slide show could not be saved");
                }

                setVisible(false);
		dispose();

	}

	private void cancelAction() {

		setVisible(false);
		dispose();

	}

	// empty mouse methods must be here so that this
        //  class meets the mouse listener interface

        public void mouseClicked(MouseEvent me) {}

        public void mouseEntered(MouseEvent me) {}

        public void mouseExited(MouseEvent me) {}

        public void mousePressed(MouseEvent me) {

		if (me.isPopupTrigger())
                   popup.show(me.getComponent(), me.getX(), me.getY());

	}

        public void mouseReleased(MouseEvent me) {

		if (me.isPopupTrigger())
                   popup.show(me.getComponent(), me.getX(), me.getY());

        }

}
