import java.io.*;
import java.util.*;

class ThumbnailThread extends Thread {

	private Vector fileNames;
	private File parentDir;
	private ImageViewer imageViewer;

	/**
	 * True if one or less ThumbnailThreads are running.
	 */
	private static boolean canRun = true;

	private static Scale scale;

	public ThumbnailThread(Vector _fileNames, File _parentDir, ImageViewer _imageViewer) {

		fileNames = _fileNames;
		parentDir = _parentDir;
		imageViewer = _imageViewer;
		scale = new Scale(imageViewer, true);
                setPriority(Thread.MIN_PRIORITY);

	}

	private synchronized void canIRun() {

		while (!canRun) {

		   try {
                        wait();
                   }
                   catch (InterruptedException e) {}

                }
                canRun = false;

	}

	public void run() {

                canIRun();

		String tNailName;
		int scaleWidth = 140;

                String parent = parentDir.getAbsolutePath() + ImageViewer.slashS;
		String dir = parent + ImagePreview.THUMBNAIL_DIR + ImageViewer.slashS;

		int len = fileNames.size();
		for (int i = 0; i < len; i++) {

		    String filename = (String)fileNames.get(i);

		    File file = new File(parent+filename); // the file to convert to a thumbnail

		    String tName = dir + filename; // get the full name of the file
                    File tFile = new File(tName);

                    // if the thumbnail doesn't exist, create it if the
                    //  file is not very small or not a GIF, JPEG, or PNG,
                    if ( !tFile.exists() ) {
                       long fileSize = file.length(); // get size of file
                       if ( (fileSize > ImagePreview.MAX_FILE_SIZE) || Utils.requiresJAI(file) )  {
		          try {
		               tNailName = scale.createThumbnail(file, tFile, scaleWidth);
	                  }
	                  catch (Exception e) {}
	               }
                    }

		}

		letRun();

	}

	private synchronized void letRun() {

                canRun = true;
                notifyAll(); // allow another ThumbnailThread to run

	}

}
