import java.io.*;
import java.net.URL;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.reflect.InvocationTargetException;


class ThumbnailViewPane extends JScrollPane
                        implements ActionListener,
                                     FocusListener,
                                       MouseListener,
                                         Runnable {

	private static String dir;
	private static ImageViewer iv;
	private static boolean cacheThumbs;
        private static File dirFile;
        private static JPanel panel;
        private static JButton[] labels;
        private static ImagePreview preview;
        private static Color selected, unselected;
        private static Dimension dimension;
        private static JFrame thumbsDialog;
        private static JScrollPane scrollPane;
        private static boolean constructed;

        private int i, ct;
        private ImageIcon icon;
	private String filename;
	private boolean added;

	public ThumbnailViewPane(ImageViewer parent, boolean _cacheThumbs) {

		super();
		iv = parent;
		cacheThumbs = _cacheThumbs;
		preview = new ImagePreview(iv);
		preview.enableThumbnailCaching(cacheThumbs);
		added = false;
		ct = 0;
		selected = new Color(0, 0, 149);
		unselected = new Color(204, 204, 204);
		dimension = new Dimension(150, 200);
                scrollPane = this;

                thumbsDialog = new JFrame("Thumbnail View");
		thumbsDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thumbsDialog.setContentPane( (JScrollPane)this );
		thumbsDialog.pack();
		thumbsDialog.setSize(570, 300);
		URL iconURL = ClassLoader.getSystemResource("icons/program.gif");
		Image img = Toolkit.getDefaultToolkit().getImage(iconURL);
                thumbsDialog.setIconImage(img);
		thumbsDialog.addWindowListener(
			     new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
                                   iv.getViewMenu().setThumbnailViewSelected(false);
                                   constructed = false;
                                   thumbsDialog = null;
                                }
		             }
		);
		thumbsDialog.setVisible(true);
		constructed = true;

	}

	/**
	 * Dummy constructor to start new thread for reading
	 *  in thumbnails
	 */
	private ThumbnailViewPane() {}

	public boolean isConstructed() {

		return constructed;

	}

	/**
	 * Sets the directory to view
	 * thumbnails for and displays
	 * the thumbnails for that directory.
	 */
	public void setDir(String _dir) {

		if ( !_dir.endsWith(ImageViewer.slashS) )
		   _dir += ImageViewer.slashS;
		dir = _dir;

		if (!added) {
		   panel = new JPanel();
		   panel.setLayout( new GridLayout(0, 3, 15, 15) );
		   getViewport().add(panel); // add the panel to the scrollpane
		   added = true;
		}
		else {
		   JViewport viewport = getViewport();
		   viewport.remove(panel);
		   panel = new JPanel();
		   panel.setLayout( new GridLayout(0, 3, 15, 15) );
		   viewport.add(panel);
		}

                panel.revalidate();

		ThumbnailViewPane tvpt = new ThumbnailViewPane();
                Thread thread = new Thread(tvpt);
	        thread.start();

	}

	public String getDir() {

		return dir;

	}

	public File getDirAsFile() {

		return dirFile;

	}

	public void closeThumbsDialog() {

		thumbsDialog.setVisible(false);
		thumbsDialog.dispose();
		thumbsDialog = null;

	}

	/**
	 * Background thread to read in thumbnails.
	 */
	public void run() {

		dirFile = new File(dir);
		showThumbs();

	}

	/**
	 * Displays thumbnails for the directory dir.
	 */
	private void showThumbs() {

		iv.setStatus("Creating Thumbnail View...");

		File[] files = dirFile.listFiles(); // get all files in directory dir
		Vector fileElems = new Vector();

		// add all image files in dir to fileElems
		Utils.getFilesAndDirs(files, fileElems);

		// sort files alphabetically
		Collections.sort(fileElems, String.CASE_INSENSITIVE_ORDER);

                final int length = fileElems.size();
                labels = new JButton[length];
                String fullFilename;
                int index, height, width;
                for (i = 0; i < length; i++) {
                    fullFilename = (String)fileElems.elementAt(i);
		    icon = preview.getImageIcon(fullFilename);
		    if (icon != null) {
		       if (thumbsDialog == null) {
		          iv.resetProgress();
		          break;
		       }
		       index = fullFilename.lastIndexOf(ImageViewer.slashS);
                       filename = fullFilename.substring(index+1); // get the file's name (w/o path)
                       doUpdate(new Runnable() {
                           public void run() {
                               labels[i] = new JButton(icon);
                               setupLabel(labels[i], filename);
                               final JPanel inner = new JPanel();
                               inner.setLayout( new BorderLayout() );
                               inner.add(labels[i], BorderLayout.NORTH);
                               JLabel label = new JLabel(filename);
                               label.setHorizontalAlignment(JLabel.CENTER);
                               inner.add(label, BorderLayout.SOUTH );
                               inner.setBorder( BorderFactory.createEtchedBorder() );
                               inner.setBackground(unselected);
                               panel.add(inner);
                               panel.revalidate();
                               scrollPane.revalidate();
                               thumbsDialog.repaint();
                               thumbsDialog.validate();
                               int progress = (i + 1) * 100;
                               iv.setProgress( progress/length );
                           }
                       });
                    }
		}

		iv.setDefaultStatus();

	}

	private void doUpdate(Runnable r) {

                try {
                    SwingUtilities.invokeAndWait(r);
                }
                catch (InvocationTargetException e1) {
                    System.err.println(e1);
                }
                catch (InterruptedException e2) {
                    System.err.println(e2);
                }

        }

	private void setupLabel(JButton label, String toolTipText) {

		label.setPreferredSize(dimension);
                label.setHorizontalAlignment(JLabel.CENTER);
                label.setOpaque(true);
                label.setToolTipText(toolTipText);
                label.addActionListener(this);
                label.addFocusListener(this);
                label.addMouseListener(this);
                label.setBorder( BorderFactory.createEtchedBorder() );
                label.setBackground(unselected);

	}

	public void actionPerformed(ActionEvent e) {

		JButton label = (JButton)e.getSource();
        	String filename = label.getToolTipText();
        	File file = new File(dir+filename);
        	iv.showImageAI(file);

	}

	public void focusGained(FocusEvent e) {

		JButton label = (JButton)e.getComponent();
		label.setBorder( BorderFactory.createLoweredBevelBorder() );
                label.setBackground(selected);

                // scroll to make this button visible in the scrollpane

                // get the button's parent
                JPanel inner = (JPanel)label.getParent();
                // get the location of the parent
                Point p1 = inner.getLocation(null);

                int x1, y1, x2, y2;
                x1 = (int)p1.getX();
                y1 = (int)p1.getY();

                JViewport view = scrollPane.getViewport();

		p1.setLocation(0, y1);
		view.setViewPosition(p1);

	}

        public void focusLost(FocusEvent e) {

        	JButton label = (JButton)e.getComponent();
        	label.setBorder( BorderFactory.createEtchedBorder() );
                label.setBackground(unselected);

        }

	// empty mouse methods must be here so that this
        //  class meets the mouse listener interface

        public void mouseClicked(MouseEvent me) {}

        public void mouseEntered(MouseEvent me) {

        	JButton label = (JButton)me.getComponent();
		label.requestFocus();

        }

        public void mouseExited(MouseEvent me) {

        	JButton label = (JButton)me.getComponent();
        	label.requestFocus();

        }

        public void mousePressed(MouseEvent me) {}

        public void mouseReleased(MouseEvent me) {}

}



