import java.io.File;
import java.util.*;

public class Utils {

    public final static String jpeg = "jpeg";
    public final static String jpg = "jpg";
    public final static String gif = "gif";
    public final static String tiff = "tiff";
    public final static String tif = "tif";
    public final static String png = "png";
    public final static String bmp = "bmp";
    public final static String allTypes = "*.gif;*.jpg;*.jpeg;*.png;*.bmp;*.tif;*.tiff;";

    /*
     * Get the extension of a file.
     */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }

    /**
     * @returns true if an image (specified by its
     * file name's extension) requires Java Advanced
     * Imaging (ie it is not a GIF, JPEG, or PNG)
     */
    public static boolean requiresJAI(File file) {

    	String ext = getExtension(file);
    	if ( ext.equals(gif) || ext.equals(jpg) ||
    	      ext.equals(jpeg) || ext.equals(png) )
    	   return false;

	return true;

    }

    public static void getFilesAndDirs(boolean useFilters , File[] files,
                                        Vector fileElems, Vector dirElems) {

		for (int i = 0; i < files.length; i++) {

        	    if ( files[i].isFile() ) {
                       // use file filters (if set in prefs)

                       if (useFilters) {
                          String ext = getExtension(files[i]);
                          if (ext != null) {
                             int index = allTypes.indexOf(ext);
                             if ( index > -1 )
                                fileElems.addElement( files[i].getName() ); // add to list model for files
                          }
                       }
                       else
                          fileElems.addElement( files[i].getName() ); // add to list model for files
                    }
                    else
                       dirElems.addElement( files[i].getName() ); // add to list model for directories

                }

    }

    public static void getFilesAndDirs(File[] files, Vector fileElems) {

		for (int i = 0; i < files.length; i++) {

        	    if ( files[i].isFile() ) {

                          String ext = getExtension(files[i]);
                          if (ext != null) {
                             int index = allTypes.indexOf(ext);
                             if ( index > -1 )
                                fileElems.addElement( files[i].getAbsolutePath() ); // add to vector for files
                          }

                    }

                }

    }

    public static Vector getImageFiles(File[] files) {

    		Vector fileS = new Vector();

		for (int i = 0; i < files.length; i++) {

        	    if ( files[i].isFile() ) {

                          String ext = getExtension(files[i]);
                          if (ext != null) {
                             int index = allTypes.indexOf(ext);
                             if ( index > -1 )
                                fileS.addElement( files[i].getName() );
                          }

                    }

                }

                return fileS;

    }

    /**
     * @returns all the files (as Strings), in sorted order in the directory
     * dirName that are one of the following types:
     * "*.gif;*.jpg;*.jpeg;*.png;*.bmp;*.tif;*.tiff"
     */
    public static Vector getFiles(String dirName) {

    	File file = new File(dirName);

        File[] files1 = file.listFiles();

        if (files1 == null)
           return null;

        Vector filesS = getImageFiles(files1);

        Collections.sort(filesS , String.CASE_INSENSITIVE_ORDER);

        return filesS;

    }

}
