import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

class ViewMenu {

	private FilePanel filePanel;
	private ScaleDialog scaleDialog;

	private OpenFrameList openFrameList;
	private ImageViewer parent;

	private JCheckBoxMenuItem thumbnailView;


	public ViewMenu(JMenuBar menuBar, boolean useFilePanel,
	                 FilePanel filePanel, OpenFrameList openFrameList,
	                  ImageViewer parent) {

		this.filePanel = filePanel;
		this.openFrameList = openFrameList;
		this.parent = parent;
		createViewMenu(menuBar, useFilePanel);

	}

	private void createViewMenu(JMenuBar menuBar, boolean useFilePanel) {

		JMenu viewMenu = new JMenu("View");
                viewMenu.setMnemonic(KeyEvent.VK_V);
                ImageViewer.createMenuItem("Scale...", KeyEvent.VK_S,
                                KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "scaleAction") );
                ImageViewer.createMenuItem("Zoom In", KeyEvent.VK_I,
                                KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "zoomInAction") );
                ImageViewer.createMenuItem("Zoom Out", KeyEvent.VK_O,
                                KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "zoomOutAction") );
                ImageViewer.createMenuItem("Zoom 1:1", KeyEvent.VK_1,
                                KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "zoom1To1Action") );
                viewMenu.addSeparator();
                ImageViewer.createMenuItem("Minimize", KeyEvent.VK_M,
                                KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "minAction") );
                ImageViewer.createMenuItem("Maximize", KeyEvent.VK_X,
                                KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "maxAction") );
                viewMenu.addSeparator();
                ImageViewer.createMenuItem("Full Screen", KeyEvent.VK_U,
                                KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK),
                                 viewMenu, new ReflectiveAction(this, "fullScreenAction") );
                if (useFilePanel) {
                   viewMenu.addSeparator();
                   ImageViewer.createMenuItem("Refresh File List", KeyEvent.VK_R,
                                   KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK),
                                    viewMenu, new ReflectiveAction(filePanel, "currDirAction") );
                   viewMenu.addSeparator();
                   thumbnailView = new JCheckBoxMenuItem("Thumbnail View", false);
		   thumbnailView.setMnemonic(KeyEvent.VK_N);
                   thumbnailView.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
                   thumbnailView.addActionListener(filePanel);
                   thumbnailView.setActionCommand(IVConstants.thumbnailViewS);
                   viewMenu.add(thumbnailView);
                }
                menuBar.add(viewMenu);

	}

	public boolean isThumbnailViewSelected() {

		return thumbnailView.isSelected();

	}

	public void setThumbnailViewSelected(boolean selected) {

		thumbnailView.setSelected(selected);

	}

	public void scaleAction() {

		if (scaleDialog == null)
		   scaleDialog = new ScaleDialog(parent);

		scaleDialog.setVisible(true);

	}

	public void fullScreenAction() {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {
                   FullScreenWindow fsWindow = new FullScreenWindow(selectedFrame);
                   fsWindow.setVisible(true);
		}

	}

	public void scaleChoiceAction() {

        	ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {

                   // get the selected item in the scaleChoice (combo box)
                   String scaleString = (String)parent.getScaleChoice().getSelectedItem();

                   ImageScalerThread imageScaler = new ImageScalerThread(parent, selectedFrame, scaleString,
                                                                          IVConstants.SCALE);
                   imageScaler.start();

                }

	}

	public void zoomInAction() { zoomAction(IVConstants.ZOOM_IN); }

	public void zoomOutAction() { zoomAction(IVConstants.ZOOM_OUT); }

	public void zoom1To1Action() { zoomAction(IVConstants.ZOOM_11); }

	/**
	 * Zoom in or out on the selected frame. <BR>
	 *  inOut == -1 (IVConstants.ZOOM_IN) if zooming in, 1 (IVConstants.ZOOM_OUT) if zooming out,
	 *    0 (IVConstants.SCALE) if scaling, or 11 (IVConstants.ZOOM_11) if zooming 1:1
	 */
	public void zoomAction(int inOut) {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();

		if (selectedFrame != null) {

		   String scaleString = selectedFrame.getScaleFactor();
                   ImageScalerThread imageScaler = new ImageScalerThread(parent, selectedFrame, scaleString,
                                                                          inOut);
		   imageScaler.start();

		}

	}

	public void minAction() {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();
		try {
	             if (selectedFrame != null)
	                selectedFrame.setIcon( !selectedFrame.isIcon() );
	        }
	        catch (PropertyVetoException pve) {}

	}

	public void maxAction() {

		ImageInternalFrame selectedFrame = openFrameList.getSelectedFrame();
		try {
	             if (selectedFrame != null)
		        selectedFrame.setMaximum( !selectedFrame.isMaximum() );
                }
                catch (PropertyVetoException pve) {}

	}

}
