import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class ViewPopupMenu extends JPopupMenu
                    implements MouseListener {

	private ImageViewer parent;
	private ViewMenu viewMenu;

	public ViewPopupMenu(ImageViewer _parent) {

		super();
		parent = _parent;
		viewMenu = parent.getViewMenu();
        	createMenu();

	}

	private void createMenuItem(String text, int mnemonic, KeyStroke accelerator,
                                     ActionListener listener) {

        	JMenuItem menuItem = new JMenuItem(text);
                menuItem.setMnemonic(mnemonic);
                menuItem.setAccelerator(accelerator);
                menuItem.addActionListener(listener);
                add(menuItem);

        }

        private void createMenu() {

		createMenuItem("Close", KeyEvent.VK_C,
                                KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(parent, "closeAction") );
                addSeparator();
        	createMenuItem("Scale...", KeyEvent.VK_S,
                                KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "scaleAction") );
                createMenuItem("Zoom In", KeyEvent.VK_I,
                                KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoomInAction") );
                createMenuItem("Zoom Out", KeyEvent.VK_O,
                                KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoomOutAction") );
                createMenuItem("Zoom 1:1", KeyEvent.VK_1,
                                KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "zoom1To1Action") );
                addSeparator();
                createMenuItem("Minimize", KeyEvent.VK_M,
                                KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "minAction") );
                createMenuItem("Maximize", KeyEvent.VK_X,
                                KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "maxAction") );
                addSeparator();
                createMenuItem("Full Screen", KeyEvent.VK_U,
                                KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK),
                                 new ReflectiveAction(viewMenu, "fullScreenAction") );

        }

        // empty mouse methods must be here so that this
        //  class meets the mouse listener interface

        public void mouseClicked(MouseEvent me) {}

        public void mouseEntered(MouseEvent me) {}

        public void mouseExited(MouseEvent me) {}

        public void mousePressed(MouseEvent me) {

		if (me.isPopupTrigger())
                   show(me.getComponent(), me.getX(), me.getY());

	}

        /**
         * Invoked when a mouse button is released - used
         * to listen for mouse clicks on the images,
         * so that the resize window can be opened.
         */
        public void mouseReleased(MouseEvent me) {

		if (me.isPopupTrigger())
                   show(me.getComponent(), me.getX(), me.getY());

        }

}
